#include <qdos.h>
#include <basic.h>
#include <string.h>
#include <stdlib.h>

/*--------------------------------------------------------------------------.
| This code is dereived from the standard c68 library (c) Dave Walker et al |
`--------------------------------------------------------------------------*/

char * getenv(const char *var)
{
    short   **p;
    char    *ptr;
    char *env = NULL;
    char *rv = NULL;
    short n = strlen(var);
    
    if(n)
    {
        /*
         *-----------------------------------------------
         *  No environment was passed.   It is therefore
         *  necessary to see if one has been set up in
         *  SuperBasic.    To do this, we search for a
         *  Name Table entry called '*ENV'.   If present.
         *  this will point to the environment.
         *-----------------------------------------------
         */
        long  *bv_sa; 
        char  *bv_nlbas, *nlp;
        struct _NT_ENT  *bv_ntbas, *ntp;
      
        env = NULL;
        _super();                                                /* supervisor mode */ 
        bv_sa    = (long *)(((long *)_sys_var)[4] + 0x68);       /* BASIC area base */
        bv_ntbas = (struct _NT_ENT *)((char *)bv_sa + bv_sa[6]); /* nametable base  */
        ntp      = (struct _NT_ENT *)((char *)bv_sa + bv_sa[7]); /* nametable ptr   */ 
        bv_nlbas = (char *)bv_sa + bv_sa[8];                     /* namelist base   */
        /*
         * Scan the name table in reverse order for m/c proc *ENV*.  
         *  set envold to basic environment string if found.
         */
        for (--ntp; ntp >= bv_ntbas; ntp--)   {           /* scan nametable      */
            if ( ntp->name_type == NT_proc_mc
             &&  ntp->var_type  == VT_null)       {       /* M/C procedure ?     */
                nlp = ntp->name_ptr + bv_nlbas;           /* point to namelist   */ 
                if (strncmp("*ENV*", nlp + 1, (size_t)5) == 0) {  /* found *ENV* ?       */ 
                    env = (char *)(*(long *)(ntp->value_ptr + 2));  /* environment pointer  */
                    break;                                /* exit loop           */  
                }
            }
        }    
        _user();                                          /* return to user mode */
    }
   
    /*
     *--------------------------------------------------
     *  By now we will have found the environment if
     *  it existed either on the stack or in SuperBasic.
     *  If it was found, then copy it.
     *--------------------------------------------------
     */
    if (env != NULL )  
    {      
        for (ptr = env; *ptr; ptr += strlen(ptr) + 1) /* copy C environment  */
        {
            if(strncmp(ptr, var, n) == 0 && *(ptr+n) == '=')
            {
                rv = strdup(ptr+n+1);
                break;
            }
        }
    }
    return rv;
}
